<?php

namespace Drupal\views_any_route\Plugin\views\field;

use Drupal\Component\Utility\Html;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Drupal\views_any_route\ViewsAnyRouteUtilities;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Defines a views field plugin.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("views_any_route_field")
 */
class ViewsAnyRouteField extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Leave empty to avoid a query on this field.
  }

  /**
   * Define the available options.
   *
   * @return array
   *   Array of available options for views_any_route form.
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['route'] = ['default' => ''];
    $options['route_params'] = ['default' => ''];
    $options['access_plugin'] = ['default' => 'views_any_route_default'];
    $options['url_plugin'] = ['default' => 'views_any_route_default'];
    $options['link_plugin'] = ['default' => 'views_any_route_default'];
    $options['button_text'] = ['default' => ''];
    $options['button_classes'] = ['default' => ''];
    $options['query_string'] = ['default' => ''];
    $options['button_options'] = ['default' => ''];
    $options['button_attributes'] = ['default' => ''];
    // @TODO
//    $options['button_prefix'] = ['default' => ['format' => NULL, 'value' => '']];
//    $options['button_suffix'] = ['default' => ['format' => NULL, 'value' => '']];
    $options['destination'] = ['default' => TRUE];
    $options['tokenize'] = ['default' => FALSE, 'bool' => TRUE];
    return $options;
  }

  /**
   * Provide the options form.
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['route'] = [
      '#type' => 'textfield',
      '#title' => t('Route'),
      '#description' => t('Drupal route string.'),
      '#default_value' => $this->options['route'],
      '#weight' => -9,
    ];
    $form['route_params'] = [
      '#type' => 'textarea',
      '#title' => t('Route Parameters'),
      '#description' => t('Drupal route parameters. Enter one parameter per line, in key=value format.'),
      '#default_value' => $this->options['route_params'],
      '#weight' => -9,
    ];
    $form['access_plugin'] = [
      '#type' => 'select',
      '#title' => t('Access Plugin'),
      '#description' => t('The plugin to use for determining access.'),
      '#options' => ViewsAnyRouteUtilities::createPluginList(),
      '#default_value' => $this->options['access_plugin'],
      '#weight' => -10,
      '#required' => TRUE,
    ];
    $form['url_plugin'] = [
      '#type' => 'select',
      '#title' => t('URL Plugin'),
      '#description' => t('The plugin to use for building the URL.'),
      '#options' => ViewsAnyRouteUtilities::createPluginList(),
      '#default_value' => $this->options['url_plugin'],
      '#weight' => -10,
      '#required' => TRUE,
    ];
    $form['link_plugin'] = [
      '#type' => 'select',
      '#title' => t('Link Plugin'),
      '#description' => t('The plugin to use for building the Link.'),
      '#options' => ViewsAnyRouteUtilities::createPluginList(),
      '#default_value' => $this->options['link_plugin'],
      '#weight' => -10,
      '#required' => TRUE,
    ];
    $form['button_text'] = [
      '#type' => 'textfield',
      '#title' => t('Button Text'),
      '#description' => t('The text that will be entered into the button.'),
      '#default_value' => $this->options['button_text'],
      '#weight' => -7,
    ];
    $form['query_string'] = [
      '#type' => 'textfield',
      '#title' => t('Query string to append to the link'),
      '#description' => t('Add the query string, without the "?" .'),
      '#default_value' => $this->options['query_string'],
      '#weight' => -6,
    ];
    $form['button_classes'] = [
      '#type' => 'textfield',
      '#title' => t('Button classes for the link - usually "button" or "btn," with additional styling classes.'),
      '#default_value' => $this->options['button_classes'],
      '#weight' => -5,
    ];
    $form['button_attributes'] = [
      '#type' => 'textarea',
      '#title' => t('Additional Button Attributes'),
      '#description' => t('Add one attribute string per line, without quotes (i.e. name=views_any_route).'),
      '#default_value' => $this->options['button_attributes'],
      '#cols' => 60,
      '#rows' => 2,
      '#weight' => -4,
    ];
    $form['destination'] = [
      '#type' => 'checkbox',
      '#title' => t('Include destination parameter?'),
      '#default_value' => $this->options['destination'],
      '#weight' => -1,
    ];
    $form['tokenize'] = $form['alter']['alter_text'];
    $form['tokenize']['#title'] = $this->t('Use tokens');
    $form['tokenize']['#description'] = $this->t('Use tokens from the current row for button/property values. See the "Replacement Patterns" below for options.');
    $form['tokenize']['#default_value'] = $this->options['tokenize'];
    $form['tokens'] = $form['alter']['help'];
    $form['tokens']['#states'] = NULL;
    $form['style_settings']['#attributes']['style'] = 'display:none;';
    $form['element_type_enable']['#attributes']['style'] = 'display:none;';
    $form['element_type']['#attributes']['style'] = 'display:none;';
    $form['element_class_enable']['#attributes']['style'] = 'display:none;';
    $form['element_class']['#attributes']['style'] = 'display:none;';
    $form['element_label_type_enable']['#attributes']['style'] = 'display:none;';
    $form['element_label_type']['#attributes']['style'] = 'display:none;';
    $form['element_label_class_enable']['#attributes']['style'] = 'display:none;';
    $form['element_label_class']['#attributes']['style'] = 'display:none;';
    $form['element_wrapper_type_enable']['#attributes']['style'] = 'display:none;';
    $form['element_wrapper_type']['#attributes']['style'] = 'display:none;';
    $form['element_wrapper_class_enable']['#attributes']['style'] = 'display:none;';
    $form['element_wrapper_class']['#attributes']['style'] = 'display:none;';
    $form['element_default_classes']['#attributes']['style'] = 'display:none;';
    $form['alter']['#attributes']['style'] = 'display:none;';
    $form['empty_field_behavior']['#attributes']['style'] = 'display:none;';
    $form['empty']['#attributes']['style'] = 'display:none;';
    $form['empty_zero']['#attributes']['style'] = 'display:none;';
    $form['hide_empty']['#attributes']['style'] = 'display:none;';
    $form['hide_alter_empty']['#attributes']['style'] = 'display:none;';
  }

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
    $tokenize = $this->options['tokenize'];
    // Load ViewsAnyRoute plugin definitions.
    $plugin_manager = \Drupal::service('plugin.manager.views_any_route');
    $plugin_definitions = $plugin_manager->getDefinitions();

    $access_plugin = $plugin_definitions[$this->options['access_plugin']]['class'];
    $url_plugin = $plugin_definitions[$this->options['url_plugin']]['class'];
    $link_plugin = $plugin_definitions[$this->options['link_plugin']]['class'];

    // Check route access

    /* @var $access_plugin \Drupal\views_any_route\Plugin\views_any_route\ViewsAnyRouteDefault */
    /* @var $url_plugin \Drupal\views_any_route\Plugin\views_any_route\ViewsAnyRouteDefault */
    /* @var $link_plugin \Drupal\views_any_route\Plugin\views_any_route\ViewsAnyRouteDefault */

    $route = $this->options['route'];
    $params_string = $tokenize ? $this->tokenizeValue($this->options['route_params'], $values->index) : $this->options['route_params'];
    $route_params = [];

    if (!empty($params_string)) {
      $route_params = ViewsAnyRouteUtilities::parameterStringToArray(PHP_EOL, $params_string);
    }

    if (!empty($route) && $access_plugin::checkAccess($route, $route_params)) {
      // OK, the route exists and we can start building the URL.
      $class_string = $tokenize ? Html::escape($this->tokenizeValue($this->options['button_classes'], $values->index)) :
        Html::escape($this->options['button_classes']);
      // Also, these are escaped later, so we aren't calling the escape function yet.
      $attrs_string = $tokenize ? $this->tokenizeValue($this->options['attrs_string'], $values->index) : $this->options['attrs_string'];
      $query_string = $tokenize ? $this->tokenizeValue($this->options['query_string'], $values->index) : $this->options['query_string'];
      $query_string = str_replace('&amp;', '&', $query_string);

      $query = !empty($query_string) ? ViewsAnyRouteUtilities::parameterStringToArray('&', $query_string) : '';
      $attrs = !empty($attrs_string) ? ViewsAnyRouteUtilities::parameterStringToArray(PHP_EOL, $attrs_string) : [];

      // Create options array.
      $options = [
        'attributes' => $attrs,
      ];
      $options['attributes']['class'] = $class_string;
      $options['query'] = $query;
      if ($this->options['destination']) {
        $destination = Url::fromRoute('<current>');
        $opts['query']['destination'] = $destination->toString();
      }

      $url = $url_plugin::generateUrl($route, $route_params, $options);
      $link_text = $tokenize ? Html::escape($this->tokenizeValue($this->options['button_text'], $values->index)) :
        Html::escape($this->options['button_text']);
      $link = $link_plugin::generateLink($url, $link_text);

      return $link->toRenderable();
    }
    else {
      return ['#markup' => ''];


//    // Get the entity/bundle type.
//    $type = explode('+', $this->options['type'], 2);
//    // If we do not have a '+', then assume we have a no-bundle entity type.
//    $entity_type = $type[0];
//    $bundle = isset($type[1]) ? $type[1] : $type[0];
//
//    // Load ViewsAnyRoute plugin definitions, and find the right one.
//    $plugin_manager = \Drupal::service('plugin.manager.views_any_route');
//    $plugin_definitions = $plugin_manager->getDefinitions();
//
//    $plugin_class = $plugin_definitions['views_any_route_default']['class'];
//    if (isset($this->options['render_plugin']) && !empty($this->options['render_plugin'])) {
//      $plugin_class = $plugin_definitions[$this->options['render_plugin']]['class'];
//    }
//    else {
//      $set_for_bundle = FALSE;
//      foreach ($plugin_definitions as $pd) {
//        // Exclude 'manual selection' special-use plugins.
//        if (empty($pd['manual_select']) || !$pd['manual_select']) {
//          if (!empty($pd['target_entity']) && $pd['target_entity'] === $entity_type) {
//            if (!empty($pd['target_bundle'])) {
//              $b = $bundle;
//              /*
//               * In certain cases, like the Group module,
//               * we need to extract the true bundle name from a
//               * hashed bundle string.
//               */
//              if (method_exists($pd['class'], 'get_bundle')) {
//                $b = $pd['class']::get_bundle($bundle);
//              }
//              if ($pd['target_bundle'] === $b) {
//                $plugin_class = $pd['class'];
//                $set_for_bundle = TRUE;
//              }
//            }
//            elseif (!$set_for_bundle) {
//              $plugin_class = $pd['class'];
//            }
//          }
//        }
//      }
//    }
//
//
//    if ($this->checkButtonAccess($plugin_definitions, $plugin_class, $entity_type, $bundle)) {
//      // Build URL Options.
//      $opts = [];
//
//      if ($this->options['destination']) {
//        $dest = Url::fromRoute('<current>');
//        $opts['query']['destination'] = $dest->toString();
//      }
//      $opts['attributes']['class'] = $this->options['tokenize'] ? $this->tokenizeValue($this->options['button_classes'], $values->index) : $this->options['button_classes'];
//
//      // Build custom attributes.
//      if ($this->options['button_attributes']) {
//        $attrs = $this->options['button_attributes'] ? $this->tokenizeValue($this->options['button_attributes'],$values->index) : $this->options['button_attributes'];
//        $attr_lines = preg_split('/$\R?^/m', $attrs);
//        foreach ($attr_lines as $line) {
//          $attr = explode('=', $line);
//          if (count($attr) === 2) {
//            $opts['attributes'][$attr[0]] = $attr[1];
//          }
//        }
//      }
//      // Build query string.
//      if ($this->options['query_string']) {
//        $q = $this->options['tokenize'] ? $this->tokenizeValue($this->options['query_string'],$values->index) : $this->options['query_string'];
//        if ($q) {
//          $qparts = explode('&', $q);
//          foreach ($qparts as $part) {
//            $p = explode('=', $part);
//            if (is_array($p) && count($p) > 1) {
//              $opts['query'][$p[0]] = $p[1];
//            }
//          }
//        }
//      }
//
//      // Get the url from the plugin and build the link.
//      if ($this->options['context']) {
//        $context = $this->options['tokenize'] ? $this->tokenizeValue($this->options['context'],$values->index) : $this->options['context'];
//        $url = $plugin_class::generateUrl($entity_type, $bundle, $opts, $context);
//      }
//      else {
//        $url = $plugin_class::generateUrl($entity_type, $bundle, $opts);
//      }
//      $text = $this->options['button_text'] ? $this->options['button_text'] : 'Add ' . $bundle;
//      $text = $this->options['tokenize'] ? $this->tokenizeValue($text,$values->index) : $text;
//      $l = Link::fromTextAndUrl(t($text), $url)->toRenderable();
//
//      if (isset($this->options['button_prefix']) || isset($this->options['button_suffix'])) {
//        if (!empty($this->options['button_prefix']['value'])) {
//          $prefix = check_markup($this->options['button_prefix']['value'], $this->options['button_prefix']['format']);
//          $prefix = $this->options['tokenize'] ? $this->tokenizeValue($prefix,$values->index) : $prefix;
//          $l['#prefix'] = $prefix;
//        }
//        if (!empty($this->options['button_suffix']['value'])) {
//          $suffix = check_markup($this->options['button_suffix']['value'], $this->options['button_suffix']['format']);
//          $suffix = $this->options['tokenize'] ? $this->tokenizeValue($suffix,$values->index) : $suffix;
//          $l['#suffix'] = $suffix;
//        }
//        return $l;
//      }
//
//      return $l;
//    }
//    else {
//      return ['#markup' => ''];
    }
  }

}
